/***************************************************************************
 *   Copyright (C) 2005 by Nicolas Brodu                                   *
 *   nicolas.brodu@free.fr                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TIME_MEASUREMENT_H
#define TIME_MEASUREMENT_H

// Time measurement is a problem on multitasking environments. To start with, we measure time spent
// within current process, NOT the system time that changes randomly with multitasking.
// But even then, there may be page faults & cache handling, plus imprecise measurement of the exact amount
// of time spent in this process (round to nearest clock tick).
// Thus some variation is expected in the measurements. However, for our use, this is probably good enough.

// If this system is posix compliant AND supports per process clocks, define this
#if defined(HAVE_WORKING_POSIX_PER_PROCESS_CLOCK)
#include <time.h>
inline int getCurrentTime(struct timespec *tp) {
    return clock_gettime(CLOCK_PROCESS_CPUTIME_ID, tp);
}
#endif

// For Linux systems with older glibc, you may try this
#if defined(HAVE_LINUX_SYSCALL)
#include <time.h>
#include <asm/unistd.h>
inline int getCurrentTime(struct timespec *tp) {
    return syscall(__NR_clock_gettime, CLOCK_PROCESS_CPUTIME_ID, tp);
}
// Last resort is to use system time
#endif

#if (!defined(HAVE_WORKING_POSIX_PER_PROCESS_CLOCK)) && (!defined(HAVE_LINUX_SYSCALL))
#include <sys/time.h>
struct timespec {
    time_t   tv_sec;
    long     tv_nsec;
};
inline int getCurrentTime(struct timespec *tp) {
    timeval tv;
    int ret = gettimeofday(&tv, 0);
    tp->tv_sec = tv->tv_sec;
    tp->tv_nsec = tv->tv_usec * 1000;
    return ret;
}
#endif


#endif
